# RequireJS
#
# Details: script including files from RequireJS(.*) method
#
# Examples
#
#	 > requirejs.rb infile.js
#	 > puts minimized content for infile.js
#
#	 > requirejs.rb infile.js outFile.min.js
#	 > Write minimized content from infile.js to outFile.min.js
# Options
#    --help          usage info
#    --compress      enable line check and compression
# 
require 'optparse'
require 'fileutils'

module RequireJS
	module Exec
		class Genetic
			def initialize(args)
				@args = args
				@options = {}
			end

			# Parses the command-line arguments and runs the executable.
			# Calls `Kernel#exit` at the end, so it never returns.
			#
			# @see #parse
			def parse!
				begin
					parse
					rescue Exception => e
						raise e if @options[:trace] || e.is_a?(SystemExit)

						$stderr.print "#{e.class}: " unless e.class == RuntimeError
						$stderr.puts "#{e.message}"
					 exit 1
				end
				exit 0
			end

			def parse
				@opts = OptionParser.new(&method(:set_opts))
				@opts.parse!(@args)

				process_result

				@options
			end

			def process_result
				input, output = @options[:input], @options[:output]
				args = @args.dup
				input ||=
					begin
						filename = args.shift
						@options[:filename] = filename || $stdin
						# open_file(args.shift, 'w') || $stdin
					end
				output ||=open_file(args.shift, 'w') || $stdout
				@options[:input], @options[:output] = input, output
			end

			def open_file(filename, flag = 'r')
				return if filename.nil?
				flag = 'wb' if @options[:unix_newlines] && flag == 'w'
				File.open(filename, flag)
			end

		end


		# The `Render RequireJS` executable.
		class Render < Genetic
			attr_reader :default_syntax

			# @param args [Array<String>] The command-line arguments
			def initialize(args)
				super
				@default_syntax = :requirejs
			end

			protected

			# Tells optparse how to parse the arguments.
			#
			# @param opts [OptionParser]

			def set_opts(opts)
				opts.banner = <<END
Usage: requirejs [options] [INPUT] [OUTPUT]

Description:
Parsing javascript for RequireJS() function and adding to script

Options:
END

				if @default_syntax == :requirejs
					opts.on('--compress',
									'Use the Uglify validation and compression.') do
						@options[:engine] = 'compress'
					end
				end
			end

			# Processes the options set by the command-line arguments,
			# and runs the RequireJS compiler appropriately.
			def process_result
				# require 'requirejs'
				super
				# @options[:params][:filename] = @options[:filename]

				begin
					input = @options[:input]
					output = @options[:output]
					document = ''
					File.open(input,'r') do |script|
						script.each_line do |line|
							requirejs = /RequireJS\(['|"|`](.*)['|"|`]\)(.*)/.match(line)
							if ( requirejs )
								include = File.join(File.dirname(input), requirejs[1])
								document += File.read(include)
							else
								document += line
							end
						end
					end
					if (@options[:engine] == 'compress')
						require 'uglifier'
						document = Uglifier.compile(document)
					end
					input.close() if input.is_a?(File)
					output.write(document)
					output.close() if output.is_a? File
				end
			end

		end
	end
end
